#!/usr/bin/env python
#
# A free (public domain), python implementation of Dijkstra's algorithm
# By: nikosapi (November 20th, 2008)
#
# Available for download at: http://nikosapi.org/software/dijkstra.zip
#

import os.path
import string
import sys

QUIET=False     # don't even print errors
VERBOSE=True    # print more information than is needed

class Dijkstra(object):
    """ A python implementation of Dijkstra's algorithm for finding the
        shortest path in a weighted graph.

        Details: http://en.wikipedia.org/wiki/Dijkstra's_algorithm
        """

    def __init__(self):
        # the distances between each node
        self.relative_distances = {}
        # the distances from the starting node (None indicates infinity)
        self.absolute_distances = {}
        # holds a dict of the shortest path to every node
        self.previous_nodes = {}
        # a list of all nodes which is used to determine
        # when the main loop should exit
        self.node_list = []
        # the current node that we are operating on
        self.current_node = None
        # the start and end nodes
        self.start_node = None
        self.end_node = None

    @property
    def nodes(self):
        # Returns the list of all the nodes in self.relative_distances
        return self.relative_distances.keys()

    def load_from_file( self, filename ):
        # Load graph data from a file, see load_from_string for more info

        if os.path.exists(filename):
            try:
                f = open(filename, 'r')
            except:
                log( 'Error opening file...', stderr=True )
                return False

            data = f.read()
            f.close()
            return self.load_from_string(data)
        else:
            log( 'File not found...', stderr=True )
            return False

    def load_from_string( self, string ):
        """ Load graph data from a file/string

            The file format is simple:
                - Comments start with a hash (#)
                - The start node is defined like so: *[start node]
                - The end node is defined like so:   &[end node]
                - The edges and their weights are defined like so:
                    [node 1] [node 2] [weight]
        """
        lines = []
        start_node = None
        end_node = None

        for i, line in enumerate(string.splitlines()):
            line = line.strip()
            if not line or line.startswith('#'):
                pass # ignore comments and blank lines
            elif line.startswith('*'):
                start_node = line.split('*',1)[1]
            elif line.startswith('&'):
                end_node = line.split('&',1)[1]
            else:
                line = line.split(' ')
                if len(line) == 3:
                    lines.append(line)
                else:
                    log( 'Syntax error on line %d: %s' % (i+1, line),
                        stderr=True )
                    return False

        distances = {}
        for node1, node2, distance in lines:
            distance = int(distance)
            if not distances.has_key(node1):
                distances[node1] = { node2: distance }
            else:
                distances[node1][node2] = distance

        if start_node is None or start_node not in distances.keys():
            log( 'No start node defined', stderr=True )
            return False
        else:
            self.start_node = self.current_node = start_node

        if end_node is None or end_node not in distances.keys():
            log( 'No end node defined', stderr=True )
            return False
        else:
            self.end_node = end_node

        self.relative_distances = distances
        self.absolute_distances = dict([ [node,None] for node in self.nodes ])
        self.previous_nodes = dict([ [node,None] for node in self.nodes ])
        self.node_list = self.nodes

        return True

    def get_distance( self, node1, node2 ):
        """ Get the _relative_ distance between two nodes
                Return None if the nodes aren't connected """

        if ( self.relative_distances.has_key(node1) and 
                self.relative_distances[node1].has_key(node2) ):
            return self.relative_distances[node1][node2]

    def get_connected_nodes( self, node ):
        """ Return a list of nodes that are connected to 'node' 
                If the node doesn't exist, return None """

        if self.relative_distances.has_key(node):
            return self.relative_distances[node].keys()

    def find_shortest( self, node_list ):
        """ The the shortest _absolute_ distance in a list of nodes 
                Returns a node name, if none are available, returns None """

        def minimum( d, k1, k2 ):
            if d[k1] < d[k1]: return k1
            else: return k2

        usable_distances = {}
        for node in node_list:
            if self.absolute_distances[node] is not None:
                usable_distances[node] = self.absolute_distances[node]

        if len(usable_distances) > 1:
            return reduce( lambda x,y: minimum( usable_distances, x, y ),
                usable_distances.keys() )
        elif len(usable_distances) == 1:
            return usable_distances.keys()[0]
        else:
            return None

    def distance_less_than( self, absolute1, absolute2 ):
        """ Checks if absolute1 is less than absolute2.
                Unlike <, this treats None as infinity.
                Returns True or False """

        if absolute2 is None:
            return True
        else:
            return absolute1 < absolute2

    def dijkstra( self ):
        """ Run Dijkstra's algorithm on data imported from load_from_file().
                Returns a dict of paths to nodes. """

        self.absolute_distances[self.current_node] = 0

        while self.node_list:
            self.current_node = self.find_shortest(self.node_list)
            self.node_list.remove(self.current_node)
            log( 'current node: %s' % self.current_node, stdout=True )

            for neighbor in self.get_connected_nodes(self.current_node):
                alt = ( self.absolute_distances[self.current_node]
                        + self.get_distance( self.current_node, neighbor ))

                log( '  found neighbor: %s (alt=%s, abs=%s)' % ( neighbor,
                    alt, self.absolute_distances[neighbor] ), stdout=True )

                if self.distance_less_than( alt,
                        self.absolute_distances[neighbor] ):

                    self.absolute_distances[neighbor] = alt
                    self.previous_nodes[neighbor] = self.current_node

        return self.previous_nodes

    def get_cost( self, path_list ):
        # find the total cost of a specific path

        sum = 0
        for i in range(len(path_list)-1):
            sum += self.relative_distances[path_list[i]][path_list[i+1]]
        return sum

    def get_shortest_path( self ):
        # Runs dijkstra() and creates a path list from data returned by it.

        d = self.dijkstra()
        shortest_path_list = [self.end_node]
        current_node = self.end_node

        while d[current_node] is not None:
            shortest_path_list.append(d[current_node])
            current_node = d[current_node]

        shortest_path_list = shortest_path_list[::-1]
        cost = self.get_cost( shortest_path_list )

        return shortest_path_list, cost

def log( msg, stdout=False, stderr=False ):
    # a simple logging function

    if QUIET:
        return

    if stdout and VERBOSE:
        print msg

    if stderr:
        sys.stderr.write(msg+'\n\r') # \n\r for more cross-platforminess
        sys.stderr.flush()

def fancy_path( path_list ):
    return string.join( path_list, ' => ' )

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print 'Usage: %s [filename]' % sys.argv[0]
    else:
        d = Dijkstra()
        if d.load_from_file(sys.argv[1]):
            shortest_path, cost = d.get_shortest_path()
            print
            print 'Shortest path:', fancy_path(shortest_path),
            print ' ( Cost:', cost, ')'
